﻿using System;
using System.Collections.Generic;
using System.Data.Entity;
using System.Linq;
using System.Linq.Expressions;
using HY.Model.Base;

namespace HY.Business
{
    public class BaseBusiness<T, V> where T : BaseModel<V>
    {
        protected readonly DbContext Context;

        protected DbSet<T> Set
        {
            get { return Context.Set<T>(); }
        }

        public BaseBusiness(DbContext context)
        {
            Context = context;
        }

        public DbSet<A> ObjectSet<A>() where A : BaseModel<V>
        {
            return Context.Set<A>();
        }


        #region 查询
        /// <summary>
        /// 根据指定条件查询
        /// </summary>
        /// <param name="predicate"></param>
        /// <returns></returns>
        public T FindByPredicate(Expression<Func<T, bool>> predicate)
        {
            return Set.SingleOrDefault(predicate);
        }



        /// <summary>
        /// 返回所有集合
        /// </summary>
        /// <returns></returns>
        protected virtual IQueryable<T> FindAll()
        {
            return Set;
        }

        /// <summary>
        /// 返回符合条件的集合
        /// </summary>
        /// <param name="predicate"></param>
        /// <returns></returns>
        protected virtual IQueryable<T> PFindCollectionByPredicate(Expression<Func<T, bool>> predicate)
        {
            return Set.Where(predicate);
        }

        /// <summary>
        /// 返回所有列表
        /// </summary>
        /// <returns></returns>
        public virtual List<T> FindAllList()
        {
            return Set.ToList();
        }

        /// <summary>
        /// 返回符合条件的集合
        /// </summary>
        /// <param name="predicate"></param>
        /// <returns></returns>
        public virtual List<T> FindListByPredicate(Expression<Func<T, bool>> predicate)
        {
            return Set.Where(predicate).ToList();
        }
        #endregion

        #region 增加
        /// <summary>
        /// 增加模型
        /// </summary>
        /// <param name="model"></param>
        protected virtual void PAdd(T model)
        {
            Set.Add(model);
        }

        /// <summary>
        /// 增加模型
        /// </summary>
        /// <param name="module"></param>
        /// <returns></returns>
        public virtual bool Add(T module)
        {
            PAdd(module);
            return Context.SaveChanges() > 0;
        }
        #endregion

        #region 删除
        /// <summary>
        /// 根据模型删除
        /// </summary>
        /// <param name="model"></param>
        protected virtual void PDeleteByModel(T model)
        {
            Set.Remove(model);
        }



        /// <summary>
        /// 根据条件删除
        /// </summary>
        /// <param name="predicate"></param>
        protected virtual void PDeleteByPredicate(Expression<Func<T, bool>> predicate)
        {
            var list = PFindCollectionByPredicate(predicate);
            PDeleteByList(list);
        }

        /// <summary>
        /// 根据列表删除
        /// </summary>
        /// <param name="list"></param>
        protected virtual void PDeleteByList(IQueryable<T> list)
        {
            foreach (var item in list)
            {
                PDeleteByModel(item);
            }
        }

        /// <summary>
        /// 根据模型删除
        /// </summary>
        /// <param name="model"></param>
        public virtual bool DeleteByModel(T model)
        {
            PDeleteByModel(model);
            return Context.SaveChanges() > 0;
        }



        /// <summary>
        /// 根据条件删除
        /// </summary>
        /// <param name="predicate"></param>
        public virtual bool DeleteByPredicate(Expression<Func<T, bool>> predicate)
        {
            PDeleteByPredicate(predicate);
            return Context.SaveChanges() > 0;
        }

        /// <summary>
        /// 根据列表删除
        /// </summary>
        /// <param name="list"></param>
        public virtual bool DeleteByList(IEnumerable<T> list)
        {
            foreach (var item in list)
            {
                PDeleteByModel(item);
            }
            return Context.SaveChanges() > 0;
        }

        #endregion

        /// <summary>
        /// 修改数据
        /// </summary>
        /// <param name="model"></param>
        /// <returns></returns>
        public virtual bool UpdateModel(T model)
        {
            return false;
        }
    }


    public class BaseBusinessInt<T> : BaseBusiness<T, int> where T : BaseModel<int>
    {
        public BaseBusinessInt(DbContext context)
            : base(context)
        {

        }

        /// <summary>
        /// 通过主键查询
        /// </summary>
        /// <param name="id">主键Id</param>
        /// <returns></returns>
        public T FindById(int id)
        {
            return FindByPredicate(p => p.Id == id);
        }

        /// <summary>
        /// 根据Id删除
        /// </summary>
        /// <param name="id"></param>
        protected virtual void PDeleteById(int id)
        {
            PDeleteByModel(FindById(id));
        }

        /// <summary>
        /// 根据Id删除
        /// </summary>
        /// <param name="id"></param>
        public virtual bool DeleteById(int id)
        {
            PDeleteByModel(FindById(id));
            return Context.SaveChanges() > 0;
        }
    }

    public class BaseBusinessString<T> : BaseBusiness<T, string> where T : BaseModel<string>
    {
        public BaseBusinessString(DbContext context)
            : base(context)
        {

        }

        /// <summary>
        /// 通过主键查询
        /// </summary>
        /// <param name="id">主键Id</param>
        /// <returns></returns>
        public T FindById(string id)
        {
            return FindByPredicate(p => p.Id == id);
        }

        /// <summary>
        /// 根据Id删除
        /// </summary>
        /// <param name="id"></param>
        protected virtual void PDeleteById(string id)
        {
            PDeleteByModel(FindById(id));
        }

        /// <summary>
        /// 根据Id删除
        /// </summary>
        /// <param name="id"></param>
        public virtual bool DeleteById(string id)
        {
            PDeleteByModel(FindById(id));
            return Context.SaveChanges() > 0;
        }
    }
}
